import xlsx = require('xlsx');
import MathUtil from './MathUtil';
import {Config, RetCode} from './Config';
import StringUtil from './StringUtil';

export default class ExcelUtil {
    /**
     * 倒序查找一个字符串种字母第一次出现的位置
     * @param str
     */
    public static getLastCharIndex(str: string) {
        let len = str.length
        if (!len || len < 1) {
            return -1
        }
        for (let i = len - 1; i >= 0; --i) {
            let ch = str[i]
            if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z')) {
                return i
            }
        }
        return -1
    }

    /**
     * 根据一个地址获取行号
     * @param address
     */
    public static getCellRowNum(address: string) {
        return parseInt(address.substring(this.getLastCharIndex(address) + 1), 10)
    }

    public static getCellColName(address: string) {
        return address.substring(0, this.getLastCharIndex(address) + 1)
    }

    /**
     * 读取表格自定义表头信息
     * @param workbookArg
     * @param sheetname
     */
    public static getExcelInfo(excelPath: string) {
        let workbook = xlsx.readFile(excelPath, {cellStyles: true, bookFiles: true})
        let sheetname = workbook.SheetNames[0];
        let worksheet = workbook.Sheets[sheetname]
        let ret = {
            keys: [],
            excelPath,
            sheetname,
            worksheet: [],
            cols: [],
            minRow: 3,
            maxRow: 2,
        }
        // console.log(excelPath, sheetname)
        for (let address in worksheet) {
            if (address[0] === '!') {
                // delete worksheet[address]     // 清除!开头的内置对象 !ref !margins !cols
                continue
            }

            // jump the value type of e and z (type: b Boolean, e Error, n Number, d Date, s Text, z Stub)
            if (worksheet[address].v == undefined ||
                worksheet[address].t == 'e' ||
                worksheet[address].t == 'z') {
                continue
            }
            ret.worksheet[address] = worksheet[address]

            let row = this.getCellRowNum(address);
            let col = this.getCellColName(address)
            // console.log('col', col, address, )
            let name = worksheet[address].v;
            if ((name == undefined || name == null) && row == 1) {
                Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, ret)} \n缺失表头名字!,是不是插入了空列？`)
            }
            if (ret.cols.findIndex((v) => v == col) == -1 && worksheet[col + '1'] && worksheet[col + '1'].v) {
                ret.cols.push(col);
            }
            if (ret.cols.length > 0 && ret.maxRow < row && name != undefined) {
                ret.maxRow = row;
            }
            if (row == 1) {
                // 读取表头参数
                let ls = name.indexOf('[')
                let le = name.indexOf(']')
                let outputclient = false
                let outputserver = false
                let type = ''
                let isUniq = false      // 重复的行
                let isArray = false     // 是否是数组
                let canZeroStr = false     // 是否允许字符串数据为'0'
                if (ls != -1 && le != -1 && le > ls + 1) {
                    let s = name.substring(ls + 1, le)
                    name = name.substring(0, ls)
                    for (let i = 0; i < s.length; i++) {
                        if (s[i] == 'c') {
                            outputclient = true
                        } else if (s[i] == 's') {
                            outputserver = true
                        } else if (s[i] == 'n') {
                            type = 'number'
                        } else if (s[i] == 't') {
                            type = 'string'
                        } else if (s[i] == 'u') { // uniq
                            isUniq = true
                        } else if (s[i] == 'a') { // array
                            isArray = true
                        } else if (s[i] == 'k') { // key
                            Config.objectKey = name
                        } else if (s[i] == 'z') {
                            canZeroStr = true
                        } else {
                            Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, ret)} \n意外的表头定义！${s}`)
                        }
                    }
                }
                if (ret.keys.findIndex(v => v == name) != -1) {
                    Config.checkState(RetCode.COL_NAME_REPEAT, `${StringUtil.getCheckString(address, ret)} \n同一个表中发现列名重复！${name}`)
                }
                ret.keys.push({
                    name,
                    outputclient,
                    outputserver,
                    type,
                    isUniq,
                    isArray,
                    canZeroStr,
                    des: worksheet[col + 2]?.v
                })
                delete worksheet[address];  // 删除表头信息
            }

        }
        return ret;
    }

    public static checkArr(value, address, headInfo) {
        let ret = {
            isOneArray: false,
            isTwoArray: false,
            isThreeArray: false,
        }
        if (MathUtil.isNumber(value)) {
            return ret;
        }
        if (value.indexOf(Config.THREEE_ARRAY_SYMBOL) != -1) {
            ret.isThreeArray = true;
            // if (value.indexOf(Config.TWO_ARRAY_SYMBOL) != -1) {
            //     if (value.indexOf(Config.ONE_ARRAY_SYMBOL) != -1) {
            //         ret.isOneArray = true;
            //         ret.isTwoArray = true;
            //         ret.isThreeArray = true;
            //     } else {
            //         Config.checkState(RetCode.NOT_FOUND_ONE_ARRAY_SYMBOL, `${StringUtil.getCheckString(address, headInfo)} \n数组分割符使用有误， 发现了三维分割符以及二维分隔符，但是未发现一维分割符！`)
            //     }
            // } else {
            //     if (value.indexOf('color') == -1) {
            //         Config.checkState(RetCode.NOT_FOUND_TWO_ARRAY_SYMBOL, `${StringUtil.getCheckString(address, headInfo)} \n数组分割符使用有误， 发现了三维分割符，但是未发现二维分割符！`)
            //     }
            // }
        }

        if (value.indexOf(Config.TWO_ARRAY_SYMBOL) != -1) {
            ret.isTwoArray = true;
            // if (value.indexOf(Config.ONE_ARRAY_SYMBOL) != -1) {
            //     ret.isOneArray = true;
            //     ret.isTwoArray = true;
            // } else {
            //     Config.checkState(RetCode.NOT_FOUND_TWO_ARRAY_SYMBOL, `${StringUtil.getCheckString(address, headInfo)} \n数组分割符使用有误， 发现了二维分割符，但是未发现一维分割符！`)
            // }
        }

        if (value.indexOf(Config.ONE_ARRAY_SYMBOL) != -1) {
            ret.isOneArray = true;
        }
        return ret;
    }

    public static tryGetArray(value: string, isMult: boolean, isNumber: boolean, canZeroStr?: boolean) {
        let ret = '';
        value = StringUtil.replaceAll(value, '<color=#', '<color=@')
        let threeArr = value.split(Config.THREEE_ARRAY_SYMBOL)
        for (let threeIndex = 0; threeIndex < threeArr.length; threeIndex++) {
            let twoArr = threeArr[threeIndex].split(Config.TWO_ARRAY_SYMBOL)
            if (isMult) {
                ret += '['
            }
            for (let twoIndex = 0; twoIndex < twoArr.length; twoIndex++) {
                let oneArr = twoArr[twoIndex].split(Config.ONE_ARRAY_SYMBOL)
                ret += '['
                for (let oneIndex = 0; oneIndex < oneArr.length; oneIndex++) {
                    let oneUnit = oneArr[oneIndex];
                    if (MathUtil.isNumber(oneUnit) && isNumber) {
                        ret += oneUnit
                    } else {
                        if ((canZeroStr && oneUnit.toString() == '0') || oneUnit.toString() != '0') {
                            ret += `'${oneUnit}'`	// 支持字符串数组
                        }
                    }
                    if (oneIndex != oneArr.length - 1) {
                        ret += ', '
                    }
                }
                ret += '], '
            }
            if (ret.substr(ret.length - 2, 2) == ', ') {
                ret = ret.substr(0, ret.length - 2)
            }
            if (isMult) {
                ret += '], '
            }
        }
        if (ret.substr(ret.length - 2, 2) == ', ') {
            ret = ret.substr(0, ret.length - 2)
        }
        if (threeArr.length > 1) {
            ret = `[${ret}]`
        }
        ret = StringUtil.replaceAll(ret, '<color=@', '<color=#')
        return ret;
    }

    public static getObj(values, isMultArr, isArray, headInfo, row) {
        let cobj = {}
        let sobj = {}
        for (let i = 0; i < values.length; i++) {
            let temp = values[i].toString()
            temp = StringUtil.replaceDoubleMid(temp, '$', mid => (parseInt(mid, 10) * Config.yuanRate).toString(), true)
            temp = StringUtil.replaceDoubleMid(temp, '￥', mid => Config.replaceStr ? Config.replaceStr : mid)
            if (headInfo.keys[i].outputserver) {
                sobj[headInfo.keys[i].name] = temp
            }
            if (headInfo.keys[i].outputclient) {
                let address = headInfo.cols[i] + row
                if (headInfo.keys[i].type == 'number' || headInfo.keys[i].type == 'number[]') {
                    if (headInfo.keys[i].isArray) {
                        headInfo.keys[i].type = 'number[]'
                        temp = this.tryGetArray(temp, isMultArr[i], true)
                        let regex: RegExp = /^[0-9\[\],"\s\-.]+$/
                        if (!regex.test(temp)) Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} \n数组内容存在字符串！`)
                    } else {
                        temp = Number(temp)
                        if (isNaN(temp)) {
                            Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} \n数字内容存在字符串！`)
                        }
                    }
                } else if (headInfo.keys[i].type == 'string' || headInfo.keys[i].type == 'string[]') {
                    if (isArray[i]) {
                        headInfo.keys[i].type = 'string[]'
                        temp = this.tryGetArray(temp, isMultArr[i], false, headInfo.keys[i].canZeroStr)
                    } else {
                        if (!headInfo.keys[i].canZeroStr && temp == '0') temp = ''
                    }
                }
                cobj[headInfo.keys[i].name] = temp
            }
        }
        return {
            c: cobj,
            s: sobj,
        }
    }

    public static creatObj(headInfo) {
        let worksheet = headInfo.worksheet
        let cobjlist = []
        let sobjlist = []
        let errorlist = [];

        for (let col = 0; col < headInfo.cols.length; col++) {
            let uniq = [];
            let colError = [];
            if (headInfo.keys[col].isUniq) {
                for (let row = headInfo.minRow; row <= headInfo.maxRow; row++) {
                    let address = headInfo.cols[col] + row
                    if (!worksheet[address]) {
                        // console.log(address);
                        Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} 单元格为空0！`)
                    }
                    if (worksheet[address].v == undefined) {
                        Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} 数据错误1！`)
                    }

                    let value = worksheet[address].v;
                    let index = MathUtil.findSame(uniq, value, 'value')
                    if (index == -1) {
                        uniq.push({address, value})
                    } else {
                        let first = uniq[index]
                        if (MathUtil.findSame(colError, first.address, 'address') == -1) {
                            colError.push(first)
                        }
                        if (MathUtil.findSame(colError, address, 'address') == -1) {
                            colError.push({address, value})
                        }
                    }
                }
            }
            if (colError.length > 0) {
                errorlist.push({name: headInfo.cols[col], list: colError})
            }
        }
        if (errorlist.length > 0) {
            let out = '';
            errorlist.forEach(error => {
                out += error.name + ' ';
                console.log(`${error.name}行`)
                error.list.forEach(unit => {
                    console.log(unit)
                })
            })
            Config.checkState(RetCode.SAME_COL_ROW_UNIQ_REPEAT, `${StringUtil.getCheckString(out, headInfo)}同一列行内容重复！`)
        }

        // 遍历整张表，对每一列进行分析
        let isMultArr: boolean[] = [];
        let isArray: boolean[] = [];
        for (let col = 0; col < headInfo.cols.length; col++) {
            let arrayNum = 0;
            if (headInfo.keys[col].isArray) {
                let isOne = 1;
                let isTwo = 0;
                let isThree = 0;
                for (let row = headInfo.minRow; row <= headInfo.maxRow; row++) {
                    let address = headInfo.cols[col] + row
                    if (!worksheet[address]) {
                        // console.log(address);
                        Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} 单元格为空1！`)
                    }
                    if (worksheet[address].v == undefined) {
                        Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} 数据错误2！`)
                    }
                    let value = worksheet[address].v;
                    let result = this.checkArr(value, address, headInfo);
                    if (result.isOneArray) {
                        isOne++
                    }
                    if (result.isTwoArray) {
                        isTwo++
                    }
                    if (result.isThreeArray) {
                        isThree++
                    }
                }
                isMultArr.push(isTwo > 0 || isThree > 0)
                isArray.push(isOne > 0)
                if (isOne > 0) {
                    arrayNum = 1
                }
                if (isTwo > 0) {
                    arrayNum = 2
                }
                if (isThree > 0) {
                    arrayNum = 3
                }
            } else {
                isMultArr.push(false)
                isArray.push(false)
            }
            headInfo.keys[col].arrayNum = arrayNum;
        }

        // 初始化
        for (let row = headInfo.minRow; row <= headInfo.maxRow; row++) {
            let values = []
            let address = ''
            // 获取一列的对象
            for (let col = 0; col < headInfo.cols.length; col++) {
                address = headInfo.cols[col] + row
                if (worksheet[address] == undefined) {
                    if (headInfo.cols[col] == 'A') {
                        let lastAddress = headInfo.cols[headInfo.cols.length - 1] + row  // 获取最后一个的地址
                        if (worksheet[lastAddress] == undefined) {
                            if (row == 2) {
                                console.log(`警告：${StringUtil.getCheckString('行' + row, headInfo)} 可能是空表！`)
                                return {
                                    sheetname: headInfo.sheetname,
                                    jsobj: cobjlist,
                                    csvobj: sobjlist,
                                    headInfo
                                }
                            } else {
                                Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString('行' + row, headInfo)} 可能存在空行！，请检查`)
                            }
                        } else {
                            Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(lastAddress, headInfo)} 数据错误3！`)
                        }
                    }
                    Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} 数据错误4！`)
                }
                let value = worksheet[address].v;
                // jump the value type of e and z (type: b Boolean, e Error, n Number, d Date, s Text, z Stub)
                if (worksheet[address].t == 'e' || worksheet[address].t == 'z') {
                    continue;
                }
                if (value.toString().indexOf(',') != -1) {
                    // if (headInfo.keys[col].outputserver) {
                    //     Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} 单元格内存在英文逗号，影响csv输出！`)
                    // } else
                    if (headInfo.keys[col].outputclient) {
                        const reg = new RegExp("[\\u4E00-\\u9FFF]+", "g");
                        if (reg.test(value)) {
                            Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} 单元格内包含中文却使用了英文逗号！`)
                        }
                    }
                }
                //最多保留两位小数
                if(typeof value === 'number' && !isNaN(value)){
                    value = parseFloat(value.toFixed(2))
                }
               values.push(value)
            }
            if (headInfo.keys.length != values.length) {
                Config.checkState(RetCode.EXCEL_DATA_ERROR, `${StringUtil.getCheckString(address, headInfo)} 数据错误5！`)
            }
            let obj = this.getObj(values, isMultArr, isArray, headInfo, row)
            cobjlist.push(obj.c)
            sobjlist.push(obj.s)
        }
        return {
            sheetname: headInfo.sheetname,
            jsobj: cobjlist,
            csvobj: sobjlist,
            headInfo
        }
    }
}