#!/usr/bin/env node
import FileUtit from './FileUtil'
import {Config, RetCode} from './Config'
import path = require('path');
import ExcelUtil from './ExcelUtil';
import yargs = require('yargs');

const outPutType = ['js','ts','csv','hsv', 'jsjson', 'tsjson', 'tsjsonmin','ts_alone','js_alone']
const argv = yargs.options({
  h: { alias: 'help' , describe: 'Show help'},
  v: { alias: 'version' , describe: 'Show version number'},
  i: { type: 'string', alias: 'input' , default: Config.inputDir, describe: 'Excel config import path'},
  o: { type: 'string', alias: 'output' , default: Config.outPutDir, describe: 'Output config export path'},
  t: { type: 'array', alias: 'type', default: outPutType, choices: outPutType, describe: 'Output config type'},
  c: { type: 'boolean', alias: 'check', default: false, describe: 'Check only not output'},
  y: { type: 'number', alias: 'yuan', default: Config.yuanRate, describe: '$num$ Rate'},
  r: { type: 'string', alias: 'replace', default: Config.replaceStr, describe: '￥string￥ replace new string'},
}).usage('Usage: excel2x [options]')
.example('excel2x -i D:\\zedu_jxb\\config -o D:\\zedu_jxb\\config\\output -t ts_alone', '')
.epilog('--------------------------------------------')
.argv;

class Main {
    public run() {
        Config.inputDir = argv.i;
        Config.outPutDir = argv.o;
        Config.yuanRate = argv.y;
        Config.replaceStr = argv.r;

        if (!FileUtit.checkDir(Config.inputDir)) {
            Config.checkState(RetCode.INPUT_DIR_IS_NOT_FOUND, `配置文件夹（inputDir）不存在 ${ Config.inputDir}`)
        }
        let excelList = FileUtit.getFileList(Config.inputDir, ['.xlsx'])
        let outPutData = []
        for (let i = 0; i < excelList.length; i++) {
            FileUtit.printlog((i + 1) / excelList.length)
            let excelPath = path.join(Config.inputDir, excelList[i])
            let headInfo = ExcelUtil.getExcelInfo(excelPath)
            let data = ExcelUtil.creatObj(headInfo)
            if (argv.t.findIndex(v => v == 'ts_alone') != -1) { FileUtit.saveTsAlone(data); }
            if (argv.t.findIndex(v => v == 'js_alone') != -1) { FileUtit.saveJsAlone(data); }
            outPutData.push(data)
        }
        if (!argv.c) {
            FileUtit.checkAndMakeDir(Config.outPutDir)
            if (argv.t.findIndex(v => v == 'js') != -1) { FileUtit.saveJs(outPutData); }
            if (argv.t.findIndex(v => v == 'ts') != -1) { FileUtit.saveTs(outPutData); }
            if (argv.t.findIndex(v => v == 'jsjson') != -1) { 
                FileUtit.saveJsJson(outPutData); 
            }
            if (argv.t.findIndex(v => v == 'tsjson') != -1) { 
                FileUtit.checkAndMakeDir(path.join(Config.outPutDir, Config.jsonOutputDir))
                FileUtit.checkAndMakeDir(path.join(Config.outPutDir, Config.tsOutputDir))
                FileUtit.saveTsJson(outPutData);
            }
            if (argv.t.findIndex(v => v == 'tsjsonmin') != -1) { 
                FileUtit.checkAndMakeDir(path.join(Config.outPutDir, Config.jsonOutputDir))
                FileUtit.checkAndMakeDir(path.join(Config.outPutDir, Config.tsOutputDir))
                FileUtit.saveTsJson(outPutData, true);
            }
            if (argv.t.findIndex(v => v == 'csv') != -1) { 
                FileUtit.checkAndMakeDir(path.join(Config.outPutDir, 'csv'))
                FileUtit.saveCsv(outPutData);
            }
    
            if (argv.t.findIndex(v => v == 'hsv') != -1) { 
                FileUtit.checkAndMakeDir(path.join(Config.outPutDir, 'hsv'))
                FileUtit.saveHsv(outPutData); 
            }
        }
        process.exit(Config.retCode);
    }
}
new Main().run();